import argparse
from github import Github, GithubException
from ruamel.yaml import YAML
from io import StringIO
import base64
from datetime import datetime
import json
import jwt
import time
import requests

class DeployUpdate:
    def __init__(self, auth_method, pat=None, app_id=None, installation_id=None, private_key=None, metadata_file_path=None):
        self.metadata_file_path = metadata_file_path
        if auth_method == 'pat':
            self.g = Github(pat)
        elif auth_method == 'app':
            self.g = self.authenticate_github_app(app_id, installation_id, private_key)
        else:
            raise ValueError("Invalid authentication method. Use 'pat' for Personal Access Token or 'app' for GitHub App.")

    def authenticate_github_app(self, app_id, installation_id, private_key):
        # Generate JWT
        time_since_epoch = int(time.time())
        payload = {
            'iat': time_since_epoch - 60,
            'exp': time_since_epoch + (10 * 60),
            'iss': app_id
        }
        jwt_token = jwt.encode(payload, private_key, algorithm='RS256')
        # Obtain installation access token
        headers = {
            'Authorization': f"Bearer {jwt_token}",
            'Accept': 'application/vnd.github+json',
            'X-GitHub-Api-Version': '2022-11-28'
        }
        response = requests.post(f"https://api.github.com/app/installations/{installation_id}/access_tokens", headers=headers)
        response.raise_for_status()  # Ensure we raise an exception for bad responses

        access_token = response.json()['token']
        return Github(login_or_token=access_token)

    @staticmethod
    def has_file_changes(old_content, new_content):
        return old_content.strip() != new_content.strip()

    def update_values_file(self, git_repo, file_path, image_key, new_version, commit_message, branch_name, source_branch):
        try:
            contents = git_repo.get_contents(file_path, ref=branch_name)
            # Use ruamel.yaml to load the YAML content.
            yaml = YAML()
            values_yaml = yaml.load(base64.b64decode(contents.content))

            keys = image_key.split('.')
            temp = values_yaml
            for key in keys[:-1]:
                if key not in temp:
                    temp[key] = {}
                temp = temp[key]
            temp[keys[-1]] = new_version

            # Use ruamel.yaml to dump the YAML content.
            new_values_yaml_stream = StringIO()
            yaml.dump(values_yaml, new_values_yaml_stream)
            new_values_yaml = new_values_yaml_stream.getvalue()


            if not self.has_file_changes(base64.b64decode(contents.content).decode('utf-8'), new_values_yaml):
                print("No changes detected, skipping update.")
                return None

            if source_branch.startswith('feature/'):
                git_repo.update_file(contents.path, commit_message, new_values_yaml, contents.sha, branch=branch_name)
                print(f"Directly updated {file_path} in {branch_name} branch.")
            else:
                target_branch = f"update-{image_key.replace('.', '-')}-{new_version}-{datetime.now().strftime('%Y%m%d%H%M%S')}"
                source_branch_ref = f"heads/{branch_name}"
                source_sha = git_repo.get_git_ref(source_branch_ref).object.sha
                git_repo.create_git_ref(ref=f"refs/heads/{target_branch}", sha=source_sha)
                git_repo.update_file(contents.path, commit_message, new_values_yaml, contents.sha, branch=target_branch)
                print(f"Updated {file_path} in new branch: {target_branch}.")
                return target_branch  
        except GithubException as e:
            print(f"Error updating values file: {e}")
            return None

    def create_pull_request(self, git_repo, title, body, head_branch, base_branch):
        try:
            pr = git_repo.create_pull(title=title, body=body, head=head_branch, base=base_branch)
            print(f"Pull request created: {pr.html_url}")
        except GithubException as e:
            print(f"Failed to create pull request: {e}")

    def get_config_for_branch(self, branch_name):
        try:
            with open(self.metadata_file_path, 'r') as file:
                metadata = json.load(file)
            if branch_name.startswith('feature/'):
                return metadata['environments']['dev']
            elif branch_name.startswith('release/'):
                return metadata['environments']['staging']
            elif branch_name == 'main':
                return metadata['environments']['production']
            else:
                raise ValueError(f"Unknown branch name: {branch_name}")
        except Exception as e:
            print(f"Error loading metadata: {e}")
            return None

    def run(self, version, source_branch, application_name):
        config = self.get_config_for_branch(source_branch)
        if not config:
            print("Failed to load configuration for the branch.")
            return
        git_repo = self.g.get_repo(config['targetRepo'])

        updated_branch = self.update_values_file(git_repo, config['targetValuesPath'], config['imageKey'], version, f"[skip ci] Update the image version of {application_name} to {version}.", config['targetBranch'], source_branch)

        if updated_branch and not source_branch.startswith('feature/'):
            self.create_pull_request(git_repo, f"[skip ci] Update the image version of {application_name} to {version}.", "[skip ci] Automated version update", updated_branch, config['targetBranch'])

def main():
    parser = argparse.ArgumentParser(description="Update Git repository values file and optionally create a pull request.")
    parser.add_argument('--auth_method', type=str, required=True, choices=['pat', 'app'], help='Authentication method: "pat" for Personal Access Token, "app" for GitHub App.')
    parser.add_argument('--pat', type=str, help='Personal Access Token for GitHub, required if auth_method is "pat".')
    parser.add_argument('--app_id', type=str, help='GitHub App ID, required if auth_method is "app".')
    parser.add_argument('--installation_id', type=str, help='GitHub App Installation ID, required if auth_method is "app".')
    parser.add_argument('--private_key', type=str, help='GitHub App Private Key as a string, required if auth_method is "app".')
    parser.add_argument('--version', type=str, required=True, help='New version to update in the values file.')
    parser.add_argument('--metadata_file_path', type=str, required=True, help='Path to the metadata configuration file.')
    parser.add_argument('--source_branch', type=str, required=True, help='Source branch name.')
    parser.add_argument('--application_name', type=str, required=True, help="Application name")

    args = parser.parse_args()

    if args.auth_method == 'pat' and not args.pat:
        parser.error("--pat is required when auth_method is 'pat'")
    elif args.auth_method == 'app' and not all([args.app_id, args.installation_id, args.private_key]):
        parser.error("All GitHub App authentication arguments are required when auth_method is 'app'")

    deploy_update = DeployUpdate(args.auth_method, pat=args.pat, app_id=args.app_id, installation_id=args.installation_id, private_key=args.private_key, metadata_file_path=args.metadata_file_path)
    deploy_update.run(args.version, args.source_branch, args.application_name)

if __name__ == '__main__':
    main()