import argparse
import os
from packaging.version import parse

class VersionManager:
    def __init__(self, path, branch, pr_title, run_number, source_branch=""):
        self.path = path
        self.branch = branch
        self.pr_title = pr_title
        self.run_number = run_number
        self.source_branch = source_branch

    @staticmethod
    def read_version_file(path):
        """Reads the version from the VERSION file"""
        version_path = os.path.join(path, "VERSION")
        with open(version_path, 'r') as file:
            version = file.read().strip()
        return version

    @staticmethod
    def write_version_file(path, version):
        """Writes the updated version to the VERSION file"""
        version_path = os.path.join(path, "VERSION")
        with open(version_path, 'w') as file:
            file.write(version)
        return version

    def increment_version(self):
        """Determines the new version based on the branch, PR title, run number, and optionally source branch."""
        version = self.read_version_file(self.path)
        ver = parse(version)
        base_version = ver.base_version 
        major, minor, patch = [int(x) for x in base_version.split('.')]
        print(major, minor, patch)

        if self.branch == "main":
            if "major:" in self.pr_title:
                new_version = f"{major + 1}.0.0"
            elif "minor:" in self.pr_title:
                new_version = f"{major}.{minor + 1}.0"
            elif "patch:" in self.pr_title:
                new_version = f"{major}.{minor}.{patch + 1}"
            else:
                new_version = f"{major}.{minor}.{patch + 1}"

        # if self.branch.startswith("feature/"):
        #     new_version = f"{base_version}-{self.run_number}"
        # elif self.branch == "main" and self.source_branch.startswith("hotfix/"):
        #     new_version = f"{major}.{minor}.{patch + 1}"
        # elif self.branch.startswith("release/"):
        #     if "major:" in self.pr_title:
        #         new_version = f"{major + 1}.0.0"
        #     elif "minor:" in self.pr_title:
        #         new_version = f"{major}.{minor + 1}.0"
        #     elif "patch:" in self.pr_title:
        #         new_version = f"{major}.{minor}.{patch + 1}"
        #     else:
        #         new_version = f"{major}.{minor}.{patch + 1}"
        else:
            new_version = version

        self.write_version_file(self.path, new_version)
        print(f"New version: {new_version}")

def main():
    parser = argparse.ArgumentParser(description='Determine and update project version.')
    parser.add_argument('--path', required=True, help='Path to the project directory.')
    parser.add_argument('--branch', required=True, help='Destination branch name.')
    parser.add_argument('--pr-title', required=False, default="", help='Pull request title.')
    parser.add_argument('--run-number', required=True, help='GitHub run number for the workflow run.')
    parser.add_argument('--source-branch', required=False, default="", help='Source branch name.')

    args = parser.parse_args()
    vm = VersionManager(args.path, args.branch, args.pr_title, args.run_number, args.source_branch)
    vm.increment_version()

if __name__ == "__main__":
    main()
